@@ -18,14 +18,14 @@ def register():
         DocumentRecord,
         DocumentLending,
         Document,
-        Communication,
+        CommunicationOutgoing,
+        CommunicationIncoming,
         CommunicationAttachment,
         DocumentConfiguration,
         DocumentLendingDocument,
         DocumentCommunicationUser,
         CommunicationLetter,
         DocumentRecordInventoryStart,
-        AddAttachmentStart,
         TRDStart,
         module='document', type_='model')
     Pool.register(
@@ -35,6 +35,5 @@ def register():
         module='document', type_='report')
     Pool.register(
         DocumentRecordInventory,
-        AddAttachment,
         TRD,
         module='document', type_='wizard')
