@@ -22,10 +22,9 @@ from trytond.transaction import Transaction
 from trytond.config import config
 
 
-__all__ = ['Communication', 'CommunicationAttachment',
-    'DocumentCommunicationUser', 'CommunicationLetter',
-    'CommunicationLetterReport', 'AddAttachmentStart',
-    'AddAttachment']
+__all__ = ['CommunicationIncoming', 'CommunicationOutgoing', 
+    'CommunicationAttachment', 'DocumentCommunicationUser',
+    'CommunicationLetter', 'CommunicationLetterReport']
 __metaclass__ = PoolMeta
 
 _STATES = {
@@ -33,17 +32,18 @@ _STATES = {
 }
 EXTENSION = '.pdf'
 
-class Communication(Workflow, ModelSQL, ModelView):
-    'Communication'
+class CommunicationIncoming(Workflow, ModelSQL, ModelView):
+    'Communication Incoming'
     __name__ = 'document.communication'
-    _rec_name = 'reference'
-    reference = fields.Char("Reference", readonly=True, select=True)
+    _rec_name = 'number'
+    number = fields.Char("Number", readonly=True, select=True)
+    reference = fields.Char("Reference", states=_STATES, select=True)
     description = fields.Char('Description', required=True, states=_STATES)
     party = fields.Many2One('party.party', 'Party', required=False,
         states=_STATES)
     users_target = fields.Many2Many('res.user-document.communication',
         'document_communication', 'user', 'User Target',
-        states=_STATES)
+        states=_STATES, required=True)
     date_communication = fields.DateTime('Date',
             readonly=True)
     comment = fields.Text('Comment', states=_STATES)
@@ -56,6 +56,11 @@ class Communication(Workflow, ModelSQL, ModelView):
         ('outgoing', 'Outgoing'),
         ('internal', 'Internal'),
         ], 'Kind', select=True, readonly=True, states=_STATES)
+    tag_location = fields.Selection([
+        ('left', 'Left'),
+        ('right', 'Right'),
+        ('center', 'Center'),
+        ], 'Tag Location', select=True, required=False, states=_STATES)
     state = fields.Selection([
         ('draft', 'Draft'),
         ('filing', 'Filing'),
@@ -70,7 +75,7 @@ class Communication(Workflow, ModelSQL, ModelView):
         ], 'Priority', required=True, select=True, states=_STATES)
     attachments = fields.One2Many('document.communication.attachment', 'communication', 
             'Attachments', states={
-                'readonly': Eval('state') != 'filing',
+                'readonly': Eval('state') != 'signed',
                 'invisible': And(
                     Not(In(Eval('_user', '0'), Eval('users_target', []))),
                     Eval('create_uid') != Eval('_user', '0'),
@@ -81,29 +86,30 @@ class Communication(Workflow, ModelSQL, ModelView):
 
     @classmethod
     def __setup__(cls):
-        super(Communication, cls).__setup__()
+        super(CommunicationIncoming, cls).__setup__()
         cls._error_messages.update({
-            'sequence_missing': ('Sequence Communication Documents is missing!'),
+            'sequence_missing': ('Sequence Communication Incoming Documents is missing!'),
             })
         cls._order.insert(0, ('date_communication', 'DESC'))
         cls._transitions |= set((
                 ('draft', 'cancelled'),
-                ('draft', 'filing'),
-                ('filing', 'signed'),
-                ('signed', 'checked'),
+                ('draft', 'signed'),
+                ('signed', 'filing'),
+                ('signed', 'draft'),
+                ('filing', 'checked'),
                 ))
         cls._buttons.update({
                 'draft': {
-                    'invisible': Eval('state') != 'filing',
-                    },
-                'checked': {
                     'invisible': Eval('state') != 'signed',
                     },
+                'checked': {
+                    'invisible': Eval('state') != 'filing',
+                    },
                 'filing': {
-                    'invisible': Eval('state') != 'draft',
+                    'invisible': Eval('state') != 'signed',
                     },
                 'signed': {
-                    'invisible': Eval('state') != 'filing',
+                    'invisible': Eval('state') != 'draft',
                     },
                 })
 
@@ -111,6 +117,10 @@ class Communication(Workflow, ModelSQL, ModelView):
     def default_company():
         return Transaction().context.get('company')
 
+    @staticmethod
+    def default_tag_location():
+        return 'right'
+
     @staticmethod
     def default_date_communication():
         return datetime.now()
@@ -138,7 +148,7 @@ class Communication(Workflow, ModelSQL, ModelView):
     @Workflow.transition('signed')
     def signed(cls, docs):
         for doc in docs:
-            doc.update_attachments_name()
+            doc.set_number()
 
     @classmethod
     @ModelView.button
@@ -152,10 +162,10 @@ class Communication(Workflow, ModelSQL, ModelView):
     @Workflow.transition('filing')
     def filing(cls, docs):
         for doc in docs:
-            doc.set_number()
+            doc.update_attachments_name()
 
     def set_number(self):
-        if self.reference:
+        if self.number:
             return
         pool = Pool()
         Sequence = pool.get('ir.sequence')
@@ -164,11 +174,178 @@ class Communication(Workflow, ModelSQL, ModelView):
         if not configuration.document_communication_sequence:
             self.raise_user_error('sequence_missing',)
         seq = configuration.document_communication_sequence.id
-        self.write([self], {'reference': Sequence.get_id(seq)})
+        self.write([self], {'number': Sequence.get_id(seq)})
 
     def update_attachments_name(self):
         for attachment in self.attachments:
-            new_name = attachment.communication.reference + '_' + attachment.name
+            new_name = attachment.communication.number + '_' + attachment.name
+            old_pathfile = os.path.join(attachment.path, attachment.name)
+            new_pathfile = os.path.join(attachment.path, new_name)
+            os.rename(old_pathfile, new_pathfile)
+            attachment.write([attachment], {
+                'name': new_name,
+            })
+
+    def get_mail_users_target(self):
+        emails = None
+        for user in self.users_target:
+            if user.email:
+                if not emails:
+                    emails = user.email
+                else:
+                    emails += ',' + user.email
+        return emails
+
+
+class CommunicationOutgoing(Workflow, ModelSQL, ModelView):
+    'Communication Outgoing'
+    __name__ = 'document.communication.outgoing'
+    _rec_name = 'number'
+    number = fields.Char("Number", readonly=True, select=True)
+    reference = fields.Char("Reference", states=_STATES, select=True)
+    description = fields.Char('Description', required=True, states=_STATES)
+    party = fields.Many2One('party.party', 'Party', required=False,
+        states=_STATES)
+    users_target = fields.Many2Many('res.user-document.communication',
+        'document_communication', 'user', 'User Target',
+        states=_STATES, required=True)
+    date_communication = fields.DateTime('Date',
+            readonly=True)
+    comment = fields.Text('Comment', states=_STATES)
+    date_checked = fields.Date('Date Checked', states={
+            'readonly': True,
+            })
+    response_time = fields.Date('Response Date', states={
+        'readonly': Eval('state') not in ['filing'],
+        })
+    tag_location = fields.Selection([
+        ('left', 'Left'),
+        ('right', 'Right'),
+        ('center', 'Center'),
+        ], 'Tag Location', select=True, states=_STATES)
+    kind = fields.Selection([
+        ('incoming', 'Incoming'),
+        ('outgoing', 'Outgoing'),
+        ('internal', 'Internal'),
+        ], 'Kind', select=True, readonly=True, states=_STATES)
+    state = fields.Selection([
+        ('draft', 'Draft'),
+        ('filing', 'Filing'),
+        ('signed', 'Signed'),
+        ('checked', 'Checked'),
+        ('cancelled', 'Cancelled'),
+        ], 'State', readonly=True, select=True)
+    priority = fields.Selection([
+        ('low', 'Low'),
+        ('middle', 'Middle'),
+        ('high', 'High'),
+        ], 'Priority', required=True, select=True, states=_STATES)
+    attachments = fields.One2Many('document.communication.attachment', 'communication', 
+            'Attachments', states={
+                'readonly': Eval('state') != 'signed',
+                'invisible': And(
+                    Not(In(Eval('_user', '0'), Eval('users_target', []))),
+                    Eval('create_uid') != Eval('_user', '0'),
+                    )
+            })
+    company = fields.Many2One('company.company', 'Company',
+            states=_STATES, required=True)
+
+    @classmethod
+    def __setup__(cls):
+        super(CommunicationOutgoing, cls).__setup__()
+        cls._error_messages.update({
+            'sequence_missing': ('Sequence Communication Documents is missing!'),
+            })
+        cls._order.insert(0, ('date_communication', 'DESC'))
+        cls._transitions |= set((
+                ('draft', 'cancelled'),
+                ('draft', 'signed'),
+                ('signed', 'filing'),
+                ('signed', 'draft'),
+                ('filing', 'checked'),
+                ))
+        cls._buttons.update({
+                'draft': {
+                    'invisible': Eval('state') != 'signed',
+                    },
+                'checked': {
+                    'invisible': Eval('state') != 'filing',
+                    },
+                'filing': {
+                    'invisible': Eval('state') != 'signed',
+                    },
+                'signed': {
+                    'invisible': Eval('state') != 'draft',
+                    },
+                })
+
+    @staticmethod
+    def default_company():
+        return Transaction().context.get('company')
+
+    @staticmethod
+    def default_tag_location():
+        return 'right'
+
+    @staticmethod
+    def default_date_communication():
+        return datetime.now()
+
+    @staticmethod
+    def default_state():
+        return 'draft'
+
+    @staticmethod
+    def default_priority():
+        return 'middle'
+
+    @staticmethod
+    def default_kind():
+        return 'incoming'
+
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('draft')
+    def draft(cls, docs):
+        pass
+
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('signed')
+    def signed(cls, docs):
+        for doc in docs:
+            doc.set_number()
+
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('checked')
+    def checked(cls, docs):
+        for doc in docs:
+            doc.write([doc], {'date_checked': date.today()})
+
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('filing')
+    def filing(cls, docs):
+        for doc in docs:
+            doc.update_attachments_name()
+
+    def set_number(self):
+        if self.number:
+            return
+        pool = Pool()
+        Sequence = pool.get('ir.sequence')
+        Configuration = pool.get('document.configuration')
+        configuration = Configuration(1)
+        if not configuration.document_communication_sequence:
+            self.raise_user_error('sequence_missing',)
+        seq = configuration.document_communication_sequence.id
+        self.write([self], {'number': Sequence.get_id(seq)})
+
+    def update_attachments_name(self):
+        for attachment in self.attachments:
+            new_name = attachment.communication.number + '_' + attachment.name
             old_pathfile = os.path.join(attachment.path, attachment.name)
             new_pathfile = os.path.join(attachment.path, new_name)
             os.rename(old_pathfile, new_pathfile)
@@ -176,19 +353,6 @@ class Communication(Workflow, ModelSQL, ModelView):
                 'name': new_name,
             })
 
-    """
-    def add_new_attachment(self, vals):
-        Attachment = Pool().get('document.communication.attachment')
-        attachment, = Attachment.create([{
-            'name': vals['name'],
-            'communication': self.id,
-            'date': vals['date'],
-            'path': vals['path'],
-            'description': vals['description'],
-            'kind': 'incoming',
-        }])
-        return attachment
-    """
 
 class CommunicationAttachment(ModelSQL, ModelView):
     'Communication Attachment'
@@ -215,6 +379,8 @@ class CommunicationAttachment(ModelSQL, ModelView):
             'parent', 'Responses')
     parent = fields.Many2One('document.communication.attachment', 'Parent',
             select=True, ondelete="RESTRICT")
+    pages = fields.Integer('Pages', states={'readonly': True})
+    indexed = fields.Boolean('Indexed', states={'readonly': True})
 
     @classmethod
     def __setup__(cls):
@@ -266,9 +432,10 @@ class CommunicationAttachment(ModelSQL, ModelView):
         """
         Set reference watermark on document PDF and return a output writeable
         """
-        ref = u'Radicado No. ' + self.communication.reference
+        ref = u'Radicado No. ' + self.communication.number
         date_ = 'Fecha: ' + str(self.create_date)[:19]
         user = 'Recibio: ' + self.create_uid.name
+        comments = '-- Recibido para su Estudio --'
 
         # read your existing PDF
         bytes_io = BytesIO(buff_file)
@@ -283,23 +450,41 @@ class CommunicationAttachment(ModelSQL, ModelView):
 
         # create a new PDF with Reportlab
         # Frame
+        rect_length, rect_width = (135, 58)
+        delta_text_y = 10
+        yrect = 62
+        ytext1 = 18
+        ytext2 = ytext1 + delta_text_y
+        ytext3 = ytext2 + delta_text_y
+        ytext4 = ytext3 + delta_text_y
+        ytext5 = ytext4 + delta_text_y
+        if self.communication.tag_location == 'left':
+            xtext = (x_dim * 0.8) + (rect_length * 0.5)
+        elif self.communication.tag_location == 'center':
+            xtext = (x_dim * 0.5) + (rect_length * 0.5)
+        elif self.communication.tag_location == 'right':
+            xtext = (x_dim * 0.2) + (rect_length * 0.5)
+        xrect = xtext + 5
+
         string_io = StringIO()
         can = canvas.Canvas(string_io, pagesize=letter)
         can.setLineWidth(0.5)
         can.setFillColor(HexColor(0xFCFCFF), alpha=0.65)
         can.setDash([3, 2, 1, 2], phase=0)
         can.setStrokeColor(HexColor(0xB3B7BA))
-        can.rect((x_dim - 145), (y_dim - 52), 135, 45, fill=1)
+        can.rect((x_dim - xrect), (y_dim - yrect), rect_length, rect_width, fill=1)
 
         # Text Frame
         can.setFont("Helvetica-Oblique", 8)
         can.setFillColorRGB(0.2, 0.2, 0.2)
-        can.drawString((x_dim - 140), (y_dim - 18), self.communication.company.party.name)
+        can.drawString((x_dim - xtext), (y_dim - ytext1), self.communication.company.party.name)
         can.setFont("Helvetica-Bold", 8)
-        can.drawString((x_dim - 140), (y_dim - 28), ref)
+        can.drawString((x_dim - xtext), (y_dim - ytext2), ref)
         can.setFont("Helvetica", 8)
-        can.drawString((x_dim - 140), (y_dim - 38), date_)
-        can.drawString((x_dim - 140), (y_dim - 48), user)
+        can.drawString((x_dim - xtext), (y_dim - ytext3), date_)
+        can.drawString((x_dim - xtext), (y_dim - ytext4), user)
+        can.setFont("Helvetica-Bold", 8)
+        can.drawString((x_dim - xtext + 8), (y_dim - ytext5), comments)
         can.save()
 
         #move to the beginning of the StringIO buffer
@@ -371,7 +556,7 @@ class AddAttachment(Wizard):
         communication = Communication(Transaction().context.get('active_id'))
         company = Company(Transaction().context.get('company'))
 
-        name_doc = communication.reference + '_' + self.start.name + EXTENSION
+        name_doc = communication.number + '_' + self.start.name + EXTENSION
         path_doc = os.path.join(full_path, name_doc)
         if os.path.exists(path_doc):
             self.raise_user_error('filename_duplicated')
@@ -379,7 +564,7 @@ class AddAttachment(Wizard):
             path_doc, 
             self.start.buff_file,
             company.party.name,
-            communication.reference,
+            communication.number,
             communication.create_uid.name,
             communication.create_date,
         )
