@@ -13,6 +13,11 @@ class DocumentConfiguration(ModelSingleton, ModelSQL, ModelView):
             required=True, domain=[
                 ('code', '=', 'document.document'),
                 ]))
+    communication_outgoing_sequence = fields.Property(
+        fields.Many2One('ir.sequence', 'Communication Outgoing Sequence',
+            required=True, domain=[
+                ('code', '=', 'document.document'),
+                ]))
     document_lending_sequence = fields.Property(
         fields.Many2One('ir.sequence', 'Document Lending Sequence',
             required=True, domain=[
